(* generated by dune *)

(** @canonical Dune_action_plugin.Import *)
module Import = Dune_action_plugin__Import

(** @canonical Dune_action_plugin.Path *)
module Path = Dune_action_plugin__Path

(** @canonical Dune_action_plugin.Protocol *)
module Protocol = Dune_action_plugin__Protocol

(** @canonical Dune_action_plugin.Sexpable_intf *)
module Sexpable_intf = Dune_action_plugin__Sexpable_intf

module Dune_action_plugin__ = struct end
[@@deprecated "this module is shadowed"]
