(* generated by dune *)

(** @canonical Dune_glob.Glob *)
module Glob = Dune_glob__Glob

(** @canonical Dune_glob.Lexer *)
module Lexer = Dune_glob__Lexer

module Dune_glob__ = struct end
[@@deprecated "this module is shadowed"]
