(* generated by dune *)

(** @canonical Dune_rpc.Client *)
module Client = Dune_rpc__Client

(** @canonical Dune_rpc.Conv *)
module Conv = Dune_rpc__Conv

(** @canonical Dune_rpc.Dbus_address *)
module Dbus_address = Dune_rpc__Dbus_address

(** @canonical Dune_rpc.Diagnostics_v1 *)
module Diagnostics_v1 = Dune_rpc__Diagnostics_v1

(** @canonical Dune_rpc.Exported_types *)
module Exported_types = Dune_rpc__Exported_types

(** @canonical Dune_rpc.Fiber_intf *)
module Fiber_intf = Dune_rpc__Fiber_intf

(** @canonical Dune_rpc.Import *)
module Import = Dune_rpc__Import

(** @canonical Dune_rpc.Menu *)
module Menu = Dune_rpc__Menu

(** @canonical Dune_rpc.Private *)
module Private = Dune_rpc__Private

(** @canonical Dune_rpc.Procedures *)
module Procedures = Dune_rpc__Procedures

(** @canonical Dune_rpc.Public *)
module Public = Dune_rpc__Public

(** @canonical Dune_rpc.Registry *)
module Registry = Dune_rpc__Registry

(** @canonical Dune_rpc.Server_notifications *)
module Server_notifications = Dune_rpc__Server_notifications

(** @canonical Dune_rpc.Sub *)
module Sub = Dune_rpc__Sub

(** @canonical Dune_rpc.Types *)
module Types = Dune_rpc__Types

(** @canonical Dune_rpc.V1 *)
module V1 = Dune_rpc__V1

(** @canonical Dune_rpc.Version_error *)
module Version_error = Dune_rpc__Version_error

(** @canonical Dune_rpc.Versioned *)
module Versioned = Dune_rpc__Versioned

(** @canonical Dune_rpc.Versioned_intf *)
module Versioned_intf = Dune_rpc__Versioned_intf

(** @canonical Dune_rpc.Where *)
module Where = Dune_rpc__Where

module Dune_rpc__ = struct end
[@@deprecated "this module is shadowed"]
